#------------------------------------------------------------------------------
#           Name: lincolnkitty.py
#         Author: Mike Edwards based on pyg_star_wars.py by Kevin Harris
#  Last Modified: 2007-02-07
#    Description: This Python/Pygame script demonstrates how to create a simple
#                 Space Invaders clone.
#------------------------------------------------------------------------------

#import the modules you need to start.  These should do for simple stuff

import random
import os, pygame
from pygame.locals import *

#it's sometimes helpful to set up a few constants for values that get used everywhere

SCREEN_WIDTH  = 800
SCREEN_HEIGHT = 600

#this is a helpful function that deals with loading images.  Use this as is 
#in your prototypes--it works well
def loadImageFile( name, useColorKey = False ):

    fullname = os.path.join( "data", name )

    try:
        image = pygame.image.load( fullname )
    except pygame.error, message:
        print "Cannot load image:", fullname
        raise SystemExit, message

    # To speed things up, convert the images to SDL's internal format.
    # If the images have an alpha channel (like with PNGs,) use it.
    if image.get_alpha is not None:
        image = image.convert_alpha()
    else:
        if useColorKey is True:
            # Use the color of the pixel located at (0,0) as our colorkey
            colorkey = image.get_at( (0,0) )
            image.set_colorkey( colorkey, RLEACCEL )
        image = image.convert()

    # Return the image
    return image

#this is a helpful function that deals with loading sounds.  Use this as is 
#in your prototypes--it works well, just like the above image loader.
def loadSoundFile( name ):

    class NoneSound:
        def play( self ): pass

    if not pygame.mixer or not pygame.mixer.get_init():
        return NoneSound()

    fullname = os.path.join( "data", name )

    try:
        sound = pygame.mixer.Sound( fullname )
    except pygame.error, message:
        print "Cannot load sound:", fullname
        raise SystemExit, message

    return sound


#this is the core of the game, where everything gets set up and processed.  It starts off with
# the set up of the game, which you can use as is for anything you do.  Then the loop will
# keep going until the conditions for stopping the loop (player quits) are met.
def main():

    # this is your set up.  You can use this as is, just be sure the change the title ;)
    random.seed()
    pygame.init()
    screen = pygame.display.set_mode( (SCREEN_WIDTH, SCREEN_HEIGHT) )
    pygame.display.set_caption( "Input" )

    # Create the background, again with our handy image loader
    background_image = loadImageFile( "background.jpg" )
    # the "rect" is the area our background takes on the screen
    background_rect = background_image.get_rect()
    # "blitting" is copying a graphic onto the screen, like pasting stickers on a wall
    screen.blit(background_image, (0,0))

    # Initialize some control variables
    running = True

    # the clock!  This keeps the game running at the pace you set.
    clock = pygame.time.Clock()

    # the "main loop"!  This is where all the action happens.  The "while" statement here tells
    # the game to keep running until something sets the "running" variable to be False.
    while running is True:
        # this tells the clock to make the game run at 60 frames per second, and no faster
        clock.tick( 60 )

        # every game or model or application will need to ask the various "controllers" what
        # the user is doing.  In pygame (and many other systems) there is a queue of events 
        # that get filled up with user input and then release.   Here we loop through that
        # queue and process the input
        for event in pygame.event.get():
            # every "event" that happens with the controller, be that hitting a key, moving
            # the mouse or closing the window, has a type associated with it.  In this loop,
            # we check for window closing (event.type == QUIT), key pushing (event.type == KEYDOWN)
            # and key releasing (event.type == KEYUP)
            if event.type == QUIT:
                #user clicked the window close icon.  Kill the game.
                running = False
            elif event.type == KEYDOWN:
                # Check for key pushes.  Esc sets running = False, and stops the main loop.
                if event.key == K_ESCAPE:
                    running = False
            elif event.type == KEYUP:
                #nothing here... yet!
                pass

        # this is very important for such a simple command!  What we've done so far has all happened in memory, and
        # now we need to update the actual hardware.  So we take what we've pasted together, and "flip" it onto
        # the screen.  Kind of like playing with a "flipbook" for animation.  
        pygame.display.flip()

        # Now that we've "flipped" the latest drawings onto the screen, the loop has finished.  Let's do it again
        # (and again and again and again...)

#this calls the 'main' function when this script is executed.  It just gets us started.  You can use this as is.
if __name__ == '__main__': 
    main()
    pygame.quit()
