#------------------------------------------------------------------------------
#           Name: lincolnkitty.py
#         Author: Mike Edwards based on pyg_star_wars.py by Kevin Harris
#  Last Modified: 2007-02-07
#    Description: This Python/Pygame script demonstrates how to create a simple
#                 Space Invaders clone.
#------------------------------------------------------------------------------

#import the modules you need to start.  These should do for simple stuff

import random
import os, pygame
from pygame.locals import *

#it's sometimes helpful to set up a few constants for values that get used everywhere

SCREEN_WIDTH  = 800
SCREEN_HEIGHT = 600

#this is a helpful function that deals with loading images.  Use this as is 
#in your prototypes--it works well
def loadImageFile( name, useColorKey = False ):

    fullname = os.path.join( "data", name )

    try:
        image = pygame.image.load( fullname )
    except pygame.error, message:
        print "Cannot load image:", fullname
        raise SystemExit, message

    # To speed things up, convert the images to SDL's internal format.
    # If the images have an alpha channel (like with PNGs,) use it.
    if image.get_alpha is not None:
        image = image.convert_alpha()
    else:
        if useColorKey is True:
            # Use the color of the pixel located at (0,0) as our colorkey
            colorkey = image.get_at( (0,0) )
            image.set_colorkey( colorkey, RLEACCEL )
        image = image.convert()

    # Return the image
    return image

#this is a helpful function that deals with loading sounds.  Use this as is 
#in your prototypes--it works well, just like the above image loader.
def loadSoundFile( name ):

    class NoneSound:
        def play( self ): pass

    if not pygame.mixer or not pygame.mixer.get_init():
        return NoneSound()

    fullname = os.path.join( "data", name )

    try:
        sound = pygame.mixer.Sound( fullname )
    except pygame.error, message:
        print "Cannot load sound:", fullname
        raise SystemExit, message

    return sound


#sprites have been in video games for almost as long as we've made them
#they're miniature graphics that get stuck, like stickers, on the big
#graphics that make up the screen.  This is a "class" that put everything
#we need to do with the player's sprite in one place.
class PlayerSprite( pygame.sprite.Sprite ):

    def __init__( self ):
        # Intialize Sprite, which is our base class
        pygame.sprite.Sprite.__init__( self )
        # uses that handy image loading function to get out image here
        self.image = loadImageFile( "lincoln.png", True)
        # the "rect" is the area on the screen covered by our image
        self.rect = self.image.get_rect()
        # this puts sprite's initial position in the middle and at the bottom
        self.rect.center = (SCREEN_WIDTH/2,SCREEN_HEIGHT)
        # these variables control how quicklt the sprite moves and in which
        # direction.  Start off at a stand still.
        self.x_velocity = 0
        self.y_velocity = 0

    def update( self ):
        # this function gets called everytime we update the sprite, which is
        # pretty much every frame.  move_ip simply moves the sprite by however
        # many pixels on the screen. 
        self.rect.move_ip( (self.x_velocity, self.y_velocity) )

        # the next few lines keep the sprite in the bounds of the screen.  These
        # kinds of statements are very useful for stopping sprites at the boundary
        # or for bouncing, reflecting, etc. off the "walls" of the screen
        if self.rect.left < 0:
            self.rect.left = 0
        elif self.rect.right > SCREEN_WIDTH:
            self.rect.right = SCREEN_WIDTH

        if self.rect.top <= SCREEN_HEIGHT/2:
            self.rect.top = SCREEN_HEIGHT/2
        elif self.rect.bottom >= SCREEN_HEIGHT:
            self.rect.bottom = SCREEN_HEIGHT

#this is another class that does much of the same things that PlayerSprite
#does, except that it does it for the enemies.  Pretty simple.  The only thing 
#that is very different is the "startx" you see in the __init__ function.  This
# lets us pick where we want the enemies to start off.
class EnemySprite( pygame.sprite.Sprite ):

    def __init__( self, startx ):
        # Intialize Sprite, which is our base class
        pygame.sprite.Sprite.__init__( self )
        self.image = loadImageFile( "kitty.png", True )
        self.rect = self.image.get_rect()
        # aha!  This is where the enemy starts off!
        self.rect.centerx = startx
        self.rect.centery = 120
        # start the motion of the enemy in some random direction
        self.x_velocity = random.randint( -5, 5 )
        self.y_velocity = random.randint( -5, 5 )
        
    def update( self ):
        self.rect.move_ip( (self.x_velocity, self.y_velocity) )

        # this is very close to the player sprites boundary functions, but this
        # bounces the enemy in the reverse direction.  Very useful for simple
        # artificial intelligence and games where bouncing is part of the mechanic
        if self.rect.left < 0 or self.rect.right > SCREEN_WIDTH:
            self.x_velocity = -(self.x_velocity)

        if self.rect.top < 0 or self.rect.bottom > SCREEN_HEIGHT/2:
            self.y_velocity = -(self.y_velocity)

        # Randomize firing, very simple AI
        fire = random.randint( 1, 60 )
        if fire == 1:
            enemy_laser_sprites.add( EnemyLaserSprite( self.rect.midbottom ) )

#yet another sprite, very similar to the previous ones
class PlayerLaserSprite( pygame.sprite.Sprite ):
    
    def __init__( self, startpos ):
        # Intialize Sprite, which is our base class
        pygame.sprite.Sprite.__init__( self )
        self.image = loadImageFile( "daisy.png", True )
        self.rect = self.image.get_rect()
        self.rect.center = startpos

    def update( self ):
        # this is simple behavior.  The sprite moves up until it hit the top of the screen
        if self.rect.bottom <= 0:
            self.kill()
        else:
            self.rect.move_ip( (0,-4) )
            

#yet another sprite, very similar to the previous ones
class EnemyLaserSprite( pygame.sprite.Sprite ):
    
    def __init__( self, startpos ):
        # Intialize Sprite, which is our base class
        pygame.sprite.Sprite.__init__( self )
        self.image = loadImageFile( "boot.png", True )
        self.rect = self.image.get_rect()
        self.rect.midtop = startpos
        
    def update( self ):
        # this is simple behavior.  The sprite moves up until it hit the bottom of the screen
        if self.rect.bottom >= SCREEN_HEIGHT:
            self.kill()
        else:
            self.rect.move_ip( (0,4) )


#this is the core of the game, where everything gets set up and processed.  It starts off with
# the set up of the game, which you can use as is for anything you do.  Then it loads up the 
# instances of each of the sprites we created above and stores them in variables.  Then we start
# the "main loop" where every frame of our game gets processed.  That means that the
# input from the controller is read, the rules based on our model are processed, and the output
# is pasted up into the view.  Then it does it again.  And again.  And AGAIN!  And it will
# keep doing it until the conditions for stopping the loop (player quits, player dies) are met.
def main():

    # this is your set up.  You can use this as is, just be sure the change the title ;)
    random.seed()
    pygame.init()
    screen = pygame.display.set_mode( (SCREEN_WIDTH, SCREEN_HEIGHT) )
    pygame.display.set_caption( "Lincoln Versus Kitties" )

    # Create the background, again with our handy image loader
    background_image = loadImageFile( "background.jpg" )
    # the "rect" is the area our background takes on the screen
    background_rect = background_image.get_rect()
    # "blitting" is copying a graphic onto the screen, like pasting stickers on a wall
    screen.blit(background_image, (0,0))

    # Initialize the player's sprite.  We make groups of sprites like this for handy
    # management of their actions and movements (more below on that).  This group
    # only has one sprite, the player.
    player_sprites = pygame.sprite.RenderUpdates()
    player_sprite = PlayerSprite()
    player_sprites.add( player_sprite )

    # these are the "laser" sprites for the player.  Nothing in here yet.
    player_laser_sprites = pygame.sprite.RenderUpdates()

    # Initialize the TIE Fighters (start the game with 3 fighters in the sprites group)
    # we'll add more as the game goes on and remove them as they get zapped.
    # By the way, the number in the EnemySprite( ) call tells the sprite where to start
    # on the horizontal axis.
    enemy_sprites = pygame.sprite.RenderUpdates()
    enemy_sprites.add( EnemySprite( 150 ) )
    enemy_sprites.add( EnemySprite( 400 ) )
    enemy_sprites.add( EnemySprite( 650 ) )

    # these are the "laser" sprites for the enemies.  Nothing here yet either.
    global enemy_laser_sprites
    enemy_laser_sprites = pygame.sprite.RenderUpdates()

    # Initialize some control variables
    running = True
    addEnemySpriteCounter = 0
    # the clock!  This keeps the game running at the pace you set.
    clock = pygame.time.Clock()

    # the "main loop"!  This is where all the action happens.  The "while" statement here tells
    # the game to keep running until something sets the "running" variable to be False.
    while running is True:
        # this tells the clock to make the game run at 60 frames per second, and no faster
        clock.tick( 60 )

        # every game or model or application will need to ask the various "controllers" what
        # the user is doing.  In pygame (and many other systems) there is a queue of events 
        # that get filled up with user input and then release.   Here we loop through that
        # queue and process the input
        for event in pygame.event.get():
            # every "event" that happens with the controller, be that hitting a key, moving
            # the mouse or closing the window, has a type associated with it.  In this loop,
            # we check for window closing (event.type == QUIT), key pushing (event.type == KEYDOWN)
            # and key releasing (event.type == KEYUP)
            if event.type == QUIT:
                #user clicked the window close icon.  Kill the game.
                running = False
            elif event.type == KEYDOWN:
                # Check for key pushes.  Esc sets running = False, and stops the main look.  The other
                # keys move the player and fire the gun.  Or, rather, they set how fast the player should
                # move once we update the player or they add a "laser" sprite to the game and play a sound
                if event.key == K_ESCAPE:
                    running = False
                elif event.key == K_LEFT:
                    player_sprite.x_velocity = -4
                elif event.key == K_RIGHT:
                    player_sprite.x_velocity = 4
                elif event.key == K_UP:
                    player_sprite.y_velocity = -4
                elif event.key == K_DOWN:
                    player_sprite.y_velocity = 4
                elif event.key == K_SPACE:
                    player_laser_sprites.add( PlayerLaserSprite( player_sprite.rect.midtop ) )
            elif event.type == KEYUP:
                # if the player releases these keys, set the velocity to equal zero.  No buttons pushed, no
                # movement.
                if event.key == K_LEFT:
                    player_sprite.x_velocity = 0
                elif event.key == K_RIGHT:
                    player_sprite.x_velocity = 0
                elif event.key == K_UP:
                    player_sprite.y_velocity = 0
                elif event.key == K_DOWN:
                    player_sprite.y_velocity = 0

        # Add another enemy.  Very simple spawning, once every 200 frames (about 3 seconds)
        addEnemySpriteCounter += 1
        if addEnemySpriteCounter >= 200:
            enemy_sprites.add( EnemySprite( 320 ) )
            addEnemySpriteCounter = 0

        # Clear everything that was drawn to the screen last time.  What these "clear" functions do is to take
        # the "rect" area of the sprites, replace the sprite's image with the background image that ought to be there.
        # Later, we'll draw the sprites onto the background again, but in their new location. 
        enemy_laser_sprites.clear( screen, background_image )
        enemy_sprites.clear( screen, background_image )
        player_laser_sprites.clear( screen, background_image)
        player_sprites.clear( screen, background_image )

        # See if any of the player's laser shots hit any of the enemies.  This makes "collision detection"
        # very easy.  pygame.sprite.groupcollide asks the game to compare the enemy_sprites group with the
        # location of the player_laser_sprites group and to delete both (that's what the two "True" parameters
        # are for) if they collide.  Then, for every hit detected, do something (nothing here yet.)
        for hit in pygame.sprite.groupcollide( enemy_sprites, player_laser_sprites, True, True):
            pass

	# See if enemies hit the player.  Pretty much the same as above, except we set running = False.  This
        # ends the game
        for hit in pygame.sprite.groupcollide( player_sprites, enemy_laser_sprites, True, True):
            running = False

        # Update all sprites.  The sprite groups call the update functions of all the sprites in them.
        # This means that the player moves (or doesn't, depending on the velocity values set above), that
        # the enemies move or bounce or whatever and that the lasers go up (for the player) or down (for
        # the enemies).
        player_sprites.update()
        player_laser_sprites.update()
        enemy_sprites.update()
        enemy_laser_sprites.update()

        # Draw everything.  This time, we replace the screen area of the sprite, which just has a background on it,
        # with the actual sprite graphic.  Easy!
        enemy_laser_sprites.draw( screen )
        player_laser_sprites.draw( screen )
        enemy_sprites.draw( screen )
        player_sprites.draw( screen )

        # this is very important for such a simple command!  What we've done so far has all happened in memory, and
        # now we need to update the actual hardware.  So we take what we've pasted together, and "flip" it onto
        # the screen.  Kind of like playing with a "flipbook" for animation.  
        pygame.display.flip()

        # Now that we've "flipped" the latest drawings onto the screen, the loop has finished.  Let's do it again
        # (and again and again and again...)

#this calls the 'main' function when this script is executed.  It just gets us started.  You can use this as is.
if __name__ == '__main__': 
    main()
    pygame.quit()
